# line protocol

# Method to create a full InfluxDB request statement (based on partial statement from client)
import uuid
from random import random, randint

def _generateClientRequest(cReq, id, time):
    # Tags first
    result = 'sid="' + str(id) + '",' + cReq

    # Fields
    # No additional fields here yet

    # Timestamp
    result += ' ' + str(_getNSTime(time))

    # Measurement
    return 'request,' + result


# Method to create a full InfluxDB response statement
def _generateServerResponse(reqID, quality, time, cpuUsage, qualityDifference):
    # Tags first
    result = ' '

    # Fields
    result += 'quality=' + str(quality) + ','
    result += 'cpuUsage=' + str(cpuUsage) + ','
    result += 'qualityDifference=' + str(qualityDifference) + ','
    result += 'requestID="' + str(reqID) + '",'
    result += 'index="' + str(uuid.uuid4()) + '"'

    # Timestamp
    result += ' ' + str(_getNSTime(time))

    # Measurement
    # print('response'+result)
    return 'response' + result


def _generateNetworkReport(sum_of_client_quality, time):
    # Measurement
    result = 'net_port_io'
    # Tags
    result += ',port_id=enps03 '
    # Fields
    result += 'RX_BYTES_PORT_M=' + str(sum_of_client_quality * 32) + ","
    result += 'TX_BYTES_PORT_M=' + str(sum_of_client_quality * 1024)
    # Timestamp
    result += ' ' + str(_getNSTime(time))

    # Measurement
    # print('network'+result)
    return result


def _generateMpegDashReport(resource, quality, time):
    # Measurement
    result = 'mpegdash_service '
    # Tags
    #None
    # Fields
    requests = randint(10, 30)
    avg_response_time = 50 + randint(0, 100) + randint(0, 10 * quality)
    peak_response_time = avg_response_time + randint(30, 60) + randint(5, 10) * quality
    result += 'cont_nav=\"' + str(resource) + "\","
    result += 'cont_rep=' + str(quality) + ','
    result += 'requests=' + str(requests) + ','
    result += 'avg_response_time=' + str(avg_response_time) + ','
    result += 'peak_response_time=' + str(peak_response_time)
    # Timestamp
    result += ' ' + str(_getNSTime(time))
    #print(result)
    return result


def _generateServerConfig(ID, location, cpu, mem, storage, time):
    # metric
    result = 'host_resource'
    # Tags
    result += ',slice_id=' + quote_wrap(ID)
    result += ',location=' + quote_wrap(location)
    result += ' '
    # Fields
    result += 'cpu=' + str(cpu)
    result += ',memory=' + quote_wrap(mem)
    result += ',storage=' + quote_wrap(storage)

    # Time
    result += ' ' + str(_getNSTime(time))

    print(result)
    return result


def _generateVMConfig(state, cpu, mem, storage, time):
    # metric
    result = 'vm_res_alloc'
    # Tags
    result += ',vm_state=' + quote_wrap(state)
    result += ' '
    # Fields
    result += 'cpu=' + str(cpu)
    result += ',memory=' + quote_wrap(mem)
    result += ',storage=' + quote_wrap(storage)

    # Time
    result += ' ' + str(_getNSTime(time))

    print(result)
    return result


def _configure_port(port_id, state, rate, time):
    # metric
    result = 'net_port_config '
    # Fields
    result += 'port_id=' + quote_wrap('enps' + port_id)
    result += ',port_state=' + quote_wrap(state)
    result += ',tx_constraint=' + quote_wrap(rate)
    result += ' '

    # Time
    result += ' ' + str(_getNSTime(time))

    print(result)
    return result

def _configure_service_function(state, max_connected_clients):
    # measurement
    result = 'mpegdash_service_config'
    # tags
    result += ',service_state='+quote_wrap(state)
    result += ' '
    # fields
    result += 'max_connected_clients='+str(max_connected_clients)

    return result

# Simulating telegraf reporting
def generate_CPU_report(requests, time):
    # Measurement
    result = 'cpu'
    # meta tag
    # We are simulating the summed CPUs, individual CPUs would have cpu=cpuNumber instead
    result += ',cpu="cpu-total"'
    result += ' '
    # field
    system = randint(0, min(100, requests*5))
    steal = randint(0, 100-system)
    idle = 100-(system+steal)
    result += 'usage_steal='+str(steal/100)
    result += ',usage_system='+str(system/100)
    result += ',usage_idle='+str(idle/100)
    result += ' '
    # Time
    result += str(_getNSTime(time))
    print(result)
    return result

def generate_mem_report(total_mem, time):
    # Measurement
    result = 'mem'
    result += ' '
    # field
    used = randint(30, 80)
    available = 100-used
    result += 'available_percent='+str(available)
    result += ',used_percent='+str(used)
    result += ',total='+str(total_mem)
    result += ' '
    # Time
    result += str(_getNSTime(time))
    print(result)
    return result

def generate_compute_node_config(slice_id, location, node_id, cpus, mem, storage, time):
    # Measurement
    result = 'compute_node_config'
    # CommonContext Tag
    result += ',slide_id='+quote_wrap(slice_id)
    # Tag
    result += ',location='+quote_wrap(location)
    result += ',comp_node_id='+quote_wrap(node_id)
    result += ' '
    # field
    result += 'cpus='+str(cpus)
    result += ',memory='+str(mem)
    result += ',storage='+str(storage)
    result += ' '
    # Time
    result += str(_getNSTime(time))
    print(result)
    return result

def generate_network_resource_config(slice_id, network_id, bandwidth, time):
    # Measurement
    result = 'network_resource_config'
    # Meta Tag
    result += ',slice_id='+quote_wrap(slice_id)
    # Tag
    result += 'network_id='+quote_wrap(network_id)
    result += ' '
    # field
    result += 'bandwidth='+str(bandwidth)
    result += ' '
    # Time
    result += str(_getNSTime(time))
    print(result)
    return result

def generate_network_interface_config(slice_id, comp_node_id, port_id, rx_constraint, tx_constraint, time):
    # Measurement
    result = 'network_interface_config'
    # Meta Tag
    result += ',slice_id'+quote_wrap(slice_id)
    # Tags
    result += ',comp_node_id='+quote_wrap(comp_node_id)
    result += ',port_id='+quote_wrap(port_id)
    result += ' '
    # field
    result += 'rx_constraint='+str(rx_constraint)
    result += ',tx_constraint='+str(tx_constraint)
    result += ' '
    # Time
    result += str(_getNSTime(time))
    print(result)
    return result





# Influx needs strings to be quoted, this provides a utility interface to do this
def quote_wrap(str):
    return "\"" + str + "\""


# InfluxDB likes to have time-stamps in nanoseconds
def _getNSTime(time):
    # Convert to nano-seconds
    return 1000000 * time
